package net.minecraft.src.buildcraft.logisticspipes.modules;

import net.minecraft.src.EntityPlayer;
import net.minecraft.src.GuiScreen;
import net.minecraft.src.IInventory;
import net.minecraft.src.ItemStack;
import net.minecraft.src.NBTTagCompound;
import net.minecraft.src.buildcraft.api.ISpecialInventory;
import net.minecraft.src.buildcraft.krapht.SimpleServiceLocator;
import net.minecraft.src.buildcraft.logisticspipes.IInventoryProvider;
import net.minecraft.src.buildcraft.logisticspipes.SidedInventoryAdapter;
import net.minecraft.src.forge.ISidedInventory;
import net.minecraft.src.krapht.InventoryUtil;
import net.minecraft.src.krapht.ItemIdentifier;

public class ModuleExtractor implements ILogisticsModule {

	private final int ticksToAction = 100;
	private int currentTick = 0;
	
	private final IInventoryProvider _invProvider;
	private final ISendRoutedItem _itemSender;
	
	public ModuleExtractor(IInventoryProvider invProvider, ISendRoutedItem itemSender) {
		_invProvider = invProvider;
		_itemSender = itemSender;
	}
	
	@Override
	public SinkReply sinksItem(ItemIdentifier item) {
		return null;
	}

	@Override
	public boolean displayGui(EntityPlayer entityplayer, GuiScreen previousGui) {
		return false;
	}

	@Override
	public ILogisticsModule getSubModule(int slot) {return null;}

	@Override
	public void readFromNBT(NBTTagCompound nbttagcompound, String prefix) {}

	@Override
	public void writeToNBT(NBTTagCompound nbttagcompound, String prefix) {}

	@Override
	public void tick() {
		if (++currentTick < ticksToAction) return;
		currentTick = 0;
		
		//Extract Item
		IInventory targetInventory = _invProvider.getInventory();
		if (targetInventory == null) return;
		
		if (targetInventory instanceof ISpecialInventory){
			ItemStack stack = ((ISpecialInventory) targetInventory).extractItem(false, _invProvider.inventoryOrientation().reverse());
			if (stack == null) return;
			if (!shouldSend(stack)) return;
			stack = ((ISpecialInventory) targetInventory).extractItem(true, _invProvider.inventoryOrientation().reverse());
			_itemSender.sendStack(stack);
			return;
		}
		
		if (targetInventory instanceof ISidedInventory){
			targetInventory = new SidedInventoryAdapter((ISidedInventory) targetInventory, _invProvider.inventoryOrientation().reverse());
		}
		
		ItemStack stackToSend;
		
		for (int i = 0; i < targetInventory.getSizeInventory(); i++){
			stackToSend = targetInventory.getStackInSlot(i);
			if (stackToSend == null) continue;
			
			if (!this.shouldSend(stackToSend)) continue;
			
			stackToSend = targetInventory.decrStackSize(i, 1);
			_itemSender.sendStack(stackToSend);
			break;
		}
	}
	
	private boolean shouldSend(ItemStack stack){
		return SimpleServiceLocator.logisticsManager.hasDestination(stack, true, _itemSender.getSourceUUID(), true);
	}
}
